#!/usr/bin/perl
#
#   convert.pl: Script for creating key and ahk files from CSV files.
#
#       Copyright (c) Yak! 2007-2008 All rights reserved.
#
#   $URL: https://yak.myhome.cx/repos/source/afxscrpt/trunk/convert.pl $
#   $Date: 2009-04-21 21:12:00 +0900 (Tue, 21 Apr 2009) $
#   $Rev: 796 $
#

use strict;
use warnings;

sub output_var
{
    my ($out, $name, $key) = @_;
    print $out <<EOF;
Afx${name}(hwnd = 0)
{
    ClipSaved := ClipboardAll
    AfxSendKey(hwnd, "$key")
    Result := Clipboard
    Clipboard := ClipSaved
    ClipSaved =
    StringTrimLeft, Result, Result, 1
    return Result
}
EOF
}

sub output_var2
{
    my ($out, $name, $var) = @_;
    $var = '$V""afx""' if $var eq '$V"afx"';
    print $out <<EOF;
Afx${name}(hwnd = 0)
{
    ClipSaved := ClipboardAll
    AfxSendCommand(hwnd, "&CLIP x$var")
    Result := Clipboard
    Clipboard := ClipSaved
    ClipSaved =
    StringTrimLeft, Result, Result, 1
    return Result
}
EOF
}

sub output_varkey
{
    my ($out, $key, $var) = @_;
    print $out <<EOF
K0000="$key&CLIP x$var"
EOF
}

sub output_command
{
    my ($out, $name, $arg, $func) = @_;
    if($arg) {
        print $out <<EOF;
Afx${name}(hwnd, arg = 0)
{
    AfxFixArg2(hwnd, arg)
    AfxSendCommand(hwnd, "&$func" " " arg)
}
EOF
    } else {
        print $out <<EOF;
Afx${name}(hwnd = 0)
{
    AfxSendCommand(hwnd, "&$func")
}
EOF
    }
}

sub output_command2
{
    my ($out, $name, $arg, $func) = @_;
    if($arg) {
        print $out <<EOF;
Afx${name}(hwnd, arg = 0)
{
    AfxFixArg2(hwnd, arg)
    AfxSendCommand(hwnd, "&$func" " " arg)
}
EOF
    } else {
        print $out <<EOF;
Afx${name}(hwnd = 0)
{
    AfxSendCommand(hwnd, "&$func")
}
EOF
    }
}

sub output_command_
{
    my ($out, $key) = @_;
    print $out <<EOF;
AfxSendCommand(hwnd, command = 0)
{
    AfxFixArg2(hwnd, command)
    _LoadInjEnvDLL()
    DllCall("injenv.dll\\InstallInjectEnvHook")
    DllCall("injenv.dll\\InjectEnvironmentVariable", "UInt", hwnd, "Str", "AFXSCRIPTVAR", "Str", command)
    AfxSendKey(hwnd, "$key")
    DllCall("injenv.dll\\UninstallInjectEnvHook")
    _UnloadInjEnvDLL()
}
EOF
}

sub output_command2_
{
    my ($out) = @_;
    print $out <<EOF;
AfxSendCommand(hwnd, command = 0)
{
    AfxFixArg2(hwnd, command)
    _LoadInjEnvDLL()
    DllCall("injenv.dll\\InstallInjectEnvHook")
    DllCall("injenv.dll\\InjectCommandA", "UInt", hwnd, "Str", command)
    DllCall("injenv.dll\\UninstallInjectEnvHook")
    _UnloadInjEnvDLL()
}

AfxSendCommandU(hwnd, command = 0)
{
    AfxFixArg2(hwnd, command)
    U2W(command_, command)
    _LoadInjEnvDLL()
    DllCall("injenv.dll\\InstallInjectEnvHook")
    DllCall("injenv.dll\\InjectCommandW", "UInt", hwnd, "Str", command_)
    DllCall("injenv.dll\\UninstallInjectEnvHook")
    _UnloadInjEnvDLL()
}
EOF
}

sub output_commandkey_
{
    my ($out, $key) = @_;
    print $out <<EOF
K0000="$key\$V"AFXSCRIPTVAR""
EOF
}

sub get_iterator
{
    my (@KEY) = ( 136..143, 151..159, 216..218, 232, 124..135 );
    my ($idx, $keyidx) = (0, 0);
    return sub {
        my ($result) = sprintf "%d%03d", $idx, $KEY[$keyidx];
        ++$keyidx;
        ($idx, $keyidx) = ($idx + 1, 0) if $keyidx == @KEY;
        return $result;
    }
}

my ($variable, $command, $ahkin, $ahkin2, $ahk2in, $ahk, $key, $doc, $ahk2) = @ARGV;

open OUTAHK, '>:crlf', $ahk;
open OUTKEY, '>:crlf', $key;
open OUTTXT, '>:crlf', $doc;
open OUTAHK2, '>:crlf', $ahk2;

print OUTAHK <<EOF;
EOF

# Copy from template
open IN, $ahkin;
while(<IN>) { print OUTAHK; s/afxscript\.ahk/afxscript2\.ahk/; print OUTAHK2; }
close IN;
open IN, $ahk2in;
while(<IN>) { print OUTAHK2; }
close IN;

my ($iterator) = get_iterator();

# Output for generic command
my ($comkey) = $iterator->();
output_command_(\*OUTAHK, $comkey);
output_command2_(\*OUTAHK2);
output_commandkey_(\*OUTKEY, $comkey);

# Output for variable
open IN, $variable;
while(<IN>) {
    s/[\r\n]+$//;
    next if /^\s*$/;
    my ($var, @name) = split /\s*,\s*/;
    my ($key) = $iterator->();
    foreach my $name (@name) {
        output_var(\*OUTAHK, $name, $key);
        output_var2(\*OUTAHK2, $name, $var);
        print OUTTXT <<EOF
    Afx$name(hwnd):
        u}N $var ̒l擾B

EOF
    }
    output_varkey(\*OUTKEY, $key, $var);
}
seek IN, 0, 0;
print OUTAHK <<EOF;
AfxGetVariable(hwnd, var = 0)
{
    AfxFixArg2(hwnd, var)
EOF
while(<IN>) {
    s/[\r\n]+$//;
    next if /^\s*$/;
    my ($var, @name) = split /\s*,\s*/;
    $var =~ s/"/""/g;
    print OUTAHK <<EOF;
    If (var == "$var")
        return Afx$name[0](hwnd)
EOF
}
print OUTAHK <<EOF;
    MsgBox, Unsupported variable \%var\% is used.
}
EOF
close IN;

print OUTAHK2 <<EOF;
AfxGetVariable(hwnd, var = 0)
{
    AfxFixArg2(hwnd, var)
    ClipSaved := ClipboardAll
    AfxSendCommand(hwnd, "&CLIP x" var)
    Result := Clipboard
    Clipboard := ClipSaved
    ClipSaved =
    StringTrimLeft, Result, Result, 1
    return Result
}

AfxGetVariableU(hwnd, var = 0)
{
    AfxFixArg2(hwnd, var)
    ClipSaved := ClipboardAll
    AfxSendCommandU(hwnd, "&CLIP x" . var)
    Transform, Result, Unicode
    Clipboard := ClipSaved
    ClipSaved =
    StringTrimLeft, Result, Result, 1
    return Result
}
EOF

# Output for command
open IN, $command;
while(<IN>) {
    s/[\r\n]+$//;
    next if /^\s*$/;
    my ($func, $arg) = split /\s*,\s*/;
    my ($funcU) = uc($func);
    output_command(\*OUTAHK, $func, $arg, $funcU);
    output_command2(\*OUTAHK2, $func, $arg, $funcU);
    print OUTTXT $arg ? <<EOF1 : <<EOF2, <<EOF3;
    Afx$func(hwnd, arg):
EOF1
    Afx$func(hwnd):
EOF2
         &$funcU sB

EOF3
}
close IN;

# Copy from template
open IN, $ahkin2;
while(<IN>) { print OUTAHK; }
close IN;

close OUTKEY;
close OUTAHK;
