;
;   win32.ahk: Win32 API helper library for AutoHotkey
;
;       Copyright (c) Yak! 2007 All rights reserved.
;
;   $URL: https://yak.myhome.cx/repos/source/afxscrpt/trunk/win32.ahk $
;   $Date: 2007-06-16 00:01:14 +0900 (Sat, 16 Jun 2007) $
;   $Rev: 399 $
;

;
; Constants
;

; Window messages
WM_USER=1024

; Status bar messages
SB_GETTEXTLENGTHA:=WM_USER+3
SB_GETTEXTA:=WM_USER+2
SB_SETTEXTA:=WM_USER+1

; Status bar drawing types
SBT_NOBORDERS=256
SBT_POPOUT=512
SBT_OWNERDRAW=0x1000
SBT_RTLREADING=1024
SBT_NORMAL=0

; Lisb box messages
LB_SETSEL=389
LB_SETCARETINDEX=414
LB_SETTOPINDEX=407
LB_GETCARETINDEX=415
LB_GETCOUNT=395
LB_GETSEL=391
LB_GETSELCOUNT=400
LB_GETSELITEMS=401
LB_GETTEXT=393
LB_GETTEXTLEN=394
LB_GETTOPINDEX=398
LB_GETITEMDATA=409
LB_SELITEMRANGE=411
LB_SELITEMRANGEEX=387
LB_ITEMFROMPOINT=425

; process security attributes
STANDARD_RIGHTS_REQUIRED=0xF0000
SYNCHRONIZE=0x100000
PROCESS_ALL_ACCESS=0x1F0FFF
PROCESS_VM_OPERATION=8
PROCESS_VM_READ=0x10
PROCESS_VM_WRITE=0x20
PROCESS_QUERY_INFORMATION=1024

; virtual memory operations
MEM_COMMIT=0x1000
MEM_RESERVE=0x2000
MEM_RELEASE=0x8000

; page protection
PAGE_READWRITE=4

;
; API wrapper functions
;

VirtualAllocEx(process, address, size, type, protect)
{
	return DllCall("VirtualAllocEx", "UInt", process, "UInt", address, "UInt", size, "UInt", type, "UInt", protect, "UInt")
}

VirtualFreeEx(process, address, size, type)
{
	return DllCall("VirtualFreeEx", "UInt", process, "UInt", address, "UInt", size, "UInt", type, "UInt")
}

ReadProcessMemory(process, address, ByRef buffer, size, ByRef read_size = -1)
{
	If read_size = -1
		return DllCall("ReadProcessMemory", "UInt", process, "UInt", address, "Str", buffer, "UInt", size, "UInt", 0, "UInt")
	Else
		return DllCall("ReadProcessMemory", "UInt", process, "UInt", address, "Str", buffer, "UInt", size, "UInt*", read_size, "UInt")
}

WriteProcessMemory(process, address, buffer, size, ByRef read_size = -1)
{
	If read_size = -1
		return DllCall("WriteProcessMemory", "UInt", process, "UInt", address, "Str", buffer, "UInt", size, "UInt", 0, "UInt")
	Else
		return DllCall("WriteProcessMemory", "UInt", process, "UInt", address, "Str", buffer, "UInt", size, "UInt*", read_size, "UInt")
}

OpenProcess(access, inherit, pid)
{
	return DllCall("OpenProcess", "UInt", access, "UInt", inherit, "UInt", pid, "UInt")
}
 
CloseHandle(handle)
{
	return DllCall("CloseHandle", "UInt", handle, "UInt")
}

GetModuleFileNameEx(arg1, arg2, ByRef arg3, arg4)
{
	return DllCall("Psapi\GetModuleFileNameExA", "UInt", arg1, "UInt", arg2, "Str", arg3, "UInt", arg4, "UInt")
}

RegisterWindowMessage(arg1)
{
	return DllCall("RegisterWindowMessage", "Str", arg1, "UInt")
}

GlobalAddAtom(arg1)
{
	return DllCall("GlobalAddAtomA", "Str", arg1, "UInt")
}

;
; API helper functions
;

GetVMProcessHandleByHWND(hwnd)
{
	global PROCESS_VM_OPERATION, PROCESS_VM_READ, PROCESS_VM_WRITE, PROCESS_QUERY_INFORMATION

	WinGet, pid, PID, ahk_id %hwnd%
	return OpenProcess(PROCESS_VM_OPERATION | PROCESS_VM_READ | PROCESS_VM_WRITE | PROCESS_QUERY_INFORMATION, 0, pid)
}

AllocProcessMemory(process, size)
{
	global MEM_COMMIT, MEM_RESERVE, PAGE_READWRITE
	return VirtualAllocEx(process, 0, size, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE)
}

FreeProcessMemory(process, address)
{
	global MEM_RELEASE
	return VirtualFreeEx(process, address, 0, MEM_RELEASE)
}

AllocWinMemReceive(hwnd, size)
{
	process:=GetVMProcessHandleByHWND(hwnd)
	address:=AllocProcessMemory(process, size)
	CloseHandle(process)
	return address
}

AllocWinMemSend(hwnd, content, terminate)
{
	If terminate
		length:=StrLen(content)+1
	Else
		length:=StrLen(content)
	process:=GetVMProcessHandleByHWND(hwnd)
	address:=AllocProcessMemory(process, length)
	WriteProcessMemory(process, address, content, length)
	CloseHandle(process)
	return address
}

ReadWinMem(hwnd, address, size)
{
	VarSetCapacity(buffer, size)
	process:=GetVMProcessHandleByHWND(hwnd)
	ReadProcessMemory(process, address, buffer, size)
	CloseHandle(process)
	return buffer
}

FreeWinMem(hwnd, address)
{
	process:=GetVMProcessHandleByHWND(hwnd)
	FreeProcessmemory(process, address)
	CloseHandle(process)
}

GetWindowExecFileName(hwnd)
{
	process:=GetVMProcessHandleByHWND(hwnd)
	VarSetCapacity(execpath, 1024)
	GetModuleFileNameEx(process, 0, execpath, 1024)
	CloseHandle(process)
	return execpath
}
